/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Scott Sheehan (ssheehan@u.washington.edu)
 * Student ID: 9820381
 * Section:    BD
 *	
 * Greeting Card Designer (Ultimate version)
 *
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <conio.h>

//-- Defines ------------------------------------------------------//

#define BTN_BGCOLOR		0
#define BTN_OBJCOLOR	1
#define BTN_ADDOBJ		2
#define BTN_ADDMSG		3
#define BTN_NEXT		4
#define BTN_SEL			5
#define BTN_FRACTAL		6

#define BGCOLORMODE		0
#define OBJCOLORMODE	1
#define ADDOBJMODE		2
#define ADDMSGMODE		3
#define NEXTMODE		4
#define SELMODE			5
#define MOVEOBJMODE		6
#define FRACTALMODE		100

#define num_buttons		7	

#define CLICK_BUTTON	0
#define CLICK_CARD		1
#define CLICK_OUTSIDE	2

#define MAX_OBJECTS		30
#define MAX_TYPES		4
#define MAX_MSGS		10

#define FILL			0

#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	100

#define SELECTED_COLOR  RED

#define HEART_SHAPE		0
#define STAR_SHAPE		1
#define FLOWER_SHAPE	2
#define CLOUD_SHAPE		3
#define FIRST_MSG		30

#define AOBJ			0
#define AMSG			1


//-- Typedef Structures -------------------------------------------//

typedef struct {
	int x,y;
} point;

typedef struct {
	int		mode;
	int		var;
	point	mPt;
	char	key;
	int		event;
} status;

typedef struct {
	int		type;
	int		color;
	int		var;
	point	oPt;
} object;

typedef struct {
	int		bgColor;
	int		numObjects;
	int		cObj;		//current object
	object	cdObject[MAX_OBJECTS];
} card;

//-- prototypes ---------------------------------------------------//


void	drawBlankScreen(void);
void	drawCard(card *theCard, int sMode);
void	drawButtons(int hilibtn);
void	drawShape(object *theObject);
void	drawHeart(object *theObject);
void	drawStar(object *theObject);
void	drawFlower(object *theObject);
void	drawCloud(object *theObject);
void	drawMessage(object *theObject);

int		classifyMouseClick(point mPt);
void	handleMouseClick(point mPt, card *theCard, status *theStatus);
void	handleButton(point mPt, card *theCard, status *theStatus);
void	handleSelect(point mPt, status *theStatus, card *theCard);
void	handleMove(point mPt, status *theStatus, card *theCard);

void	addShape(point mPt, card *theCard, status *theStatus);
void	addMsg(point mPt, card *theCard, status *theStatus);

void	initCard(card *theCard);
void	initObj(object *theObject, point mPt, int type);

void	drawFractalTree(void);
void	drawFractalTriangle(void);


//-- Source Code --------------------------------------------------//

int main(void) {
	status	theStatus;
	card	theCard;
	
	int hilibtn = -1; 
	
	time_t t; /*needed only when initializing a random number generator*/
	
	/* INITIALIZATIONS */
	srand (time(&t));
	
	initCard(&theCard);
	theStatus.mode	= ADDOBJMODE;

	GP142_open();           // Open and init the GP142 Graphics Window      
	GP142_show_text(0);     // Hide the text window                         
	GP142_logging(LOG_OFF); // Change to LOG_ON if you want it.             
	
	GP142_clear();	        // This will clear the window & paint it white. 
	GP142_animate(ANI_RUN); // Enable periodic events                       
	
	// The main event loop:
	
	theStatus.var = 1;

	while (theStatus.mode != -1) {  /*this is the famous "event loop" */
		if(theStatus.mode != FRACTALMODE) {	
			drawBlankScreen();
			drawCard(&theCard,theStatus.mode);
			drawButtons(hilibtn);	  /* Show the buttons...*/ 
		} else {
			if(theStatus.var) {
				drawBlankScreen();
				if(rand()%2) {
					drawFractalTriangle();
				} else {
					drawFractalTree();
				}
				theStatus.var = 0;
			}
		}

		/*Wait for something to happen */
		theStatus.event = GP142_await_event(&theStatus.mPt.x, 
								  &theStatus.mPt.y, 
								  &theStatus.key);
		
		switch(theStatus.event) {
			case GP142_QUIT:
				theStatus.mode = -1;        
				break;
			case GP142_KBD:  /*A key was pressed */
				break;
			case GP142_MOUSE:
				if(theStatus.mode==FRACTALMODE) {
					theStatus.var = 1;
				} else {
					handleMouseClick(theStatus.mPt,&theCard,&theStatus);	
				}
				break;
			case GP142_PERIODIC:
				/*An events has occurred simply because some time has passed,*/
				/*not currently used in the program*/		
				break;
			default:
				
				break;
				
		} 
		
	} 
	
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  

//-- Function code ------------------------------------------------//

void drawBlankScreen(void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,-GP142_XMAX,-GP142_YMAX,GP142_XMAX, 
						GP142_YMAX,FILL);
} 

void drawCard(card *theCard, int sMode) {
	int font_size = 14;
	int i;

	GP142_rectangleXY(theCard->bgColor,PAGE_LLX, PAGE_LLY,
		PAGE_LLX+PAGE_WIDTH, PAGE_LLY+PAGE_HEIGHT,FILL);
	
	for(i=0;i<=(theCard->numObjects);i++) {
		if ( (theCard->cObj >= 0) ) {
			drawShape(&theCard->cdObject[i]);
		}
	}

	switch(sMode) {
		case BGCOLORMODE:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"This will change the background color");
			break;
		case OBJCOLORMODE:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"This will change the current object's color");
			break;
		case ADDOBJMODE:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"Click on the card to add a shape");
			break;
		case ADDMSGMODE:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"Click on the card to add a message");
			break;
		case MOVEOBJMODE:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"Click on the card to move the selected object");
			break;
		case SELMODE:
			GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size-3,
				"Object selected. Click button to change shape.");
			break;
		default:
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
				"Status error -- notify programmer");
			break;
	}	
}

void drawButtons(int hilibtn) {
	char b_labels[num_buttons][25] = {
		"Background color",
		"    Object Color",
		"       Add Shape",
		"     Add Message",
		"    Next, please",
		"   Select Object",
		"      Fractals?!"
	};
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < num_buttons; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 1);
		
		
		label_y = y - BUTTON_HEIGHT/2;
		
		if (i == hilibtn) {
			GP142_printfXY(SELECTED_COLOR,text_offset,label_y,
							font_size,b_labels[i]);	
		} else {
			GP142_printfXY(BLACK,text_offset,label_y,font_size,
							b_labels[i]);	
		}
	}	
}

void drawShape(object *theObject) {

	switch ( theObject->type ) {
		case HEART_SHAPE:
			drawHeart(theObject);			break;
		case STAR_SHAPE:
			drawStar(theObject);			break;
		case FLOWER_SHAPE:
			drawFlower(theObject);			break;
		case CLOUD_SHAPE:
			drawCloud(theObject);			break;
		default: //should be a message
			drawMessage(theObject);
			break;
	} 
}


#define HEART_WIDTH 35
void drawHeart(object *theObject) {
	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY(theObject->color, theObject->oPt.x - h + 5, 
									 theObject->oPt.y, h);
	GP142_circleXY(theObject->color, theObject->oPt.x + h - 5, 
									 theObject->oPt.y, h);
	GP142_triangleXY(theObject->color, theObject->oPt.x- h*2 +5, 
					theObject->oPt.y-5,theObject->oPt.x + h*2 -5, 
					theObject->oPt.y-5,theObject->oPt.x, 
					theObject->oPt.y - h*2 -5, FILL);
}

#define STARPOINTS		5
#define STARRADIUS		40.0
#define PI				3.14159265359
#define TWOPIOVERFIVE	1.25663706144			//72 degrees
void drawStar(object *theObject) {
	int i;
	double angle;
	int x1,y1,x2,y2;

	theObject->var += 5;

	angle = ((double)(theObject->var)/180.0)*PI;
	x1 = (theObject->oPt.x) + 
			(int) ( (cos(angle)) * STARRADIUS);
	y1 = (theObject->oPt.y) + 
			(int) ( (sin(angle)) * STARRADIUS);
	
	for(i=1;i<=5;i++) {
		x2 = (theObject->oPt.x) + 
			(int) ((cos(angle+2.0*TWOPIOVERFIVE*(double)i)) * STARRADIUS);
		y2 = (theObject->oPt.y) + 
			(int) ((sin(angle+2.0*TWOPIOVERFIVE*(double)i)) * STARRADIUS);
		GP142_lineXY (theObject->color, x1, y1, x2, y2, 2);
		x1 = x2;
		y1 = y2;
	}
}

void drawMessage(object *theObject) {
	char messages[MAX_MSGS][25] = {
		"Happy Valentine's Day",
		"Be Mine",
		"I Love You",
		"Roses are Red",
		"Violets are Blue",
	};

	GP142_printfXY(theObject->color,theObject->oPt.x,theObject->oPt.y,
					9+rand()%5,messages[theObject->type-FIRST_MSG]);
}

#define STEMLENGTH	40
#define RAD			10
void drawFlower(object *theObject) {
	int x,y,px,py,r,i,c;
	double angle;

	theObject->var += 5;

	x = theObject->oPt.x;
	y = theObject->oPt.y;
	r = (int)(double)RAD*1.1;
	
	GP142_lineXY(GREEN,x,y,x,y-STEMLENGTH,2);

	c = ((theObject->color)+1)%MAX_COLORS;
	
	for(i=0;i<=4;i++) {
		angle = ((double)(theObject->var))/180.0*PI;
		angle += (PI/2.5)*(double)i;
		px = x + (int) (cos(angle)*(double)RAD*1.5);
		py = y + (int) (sin(angle)*(double)RAD*1.5);
		GP142_ovalXY(c,px-RAD,py-RAD,px+RAD,py+RAD,FILL);
	}

	GP142_ovalXY(theObject->color,x-r,y-r,x+r,y+r,FILL);

}

#define CR		18	//cloud diameter
#define CD		6	//another define
void drawCloud(object *theObject) {
	int x,y;

	x = theObject->oPt.x;
	y = theObject->oPt.y;
	
	GP142_ovalXY(theObject->color,x-CR/2,y-CR/2,
								  x+CR/2,y+CR/2,FILL);

	GP142_ovalXY(theObject->color,x-rand()%3,y-rand()%3,
								  x+CR+rand()%3,y+CR+rand()%3,FILL);
	GP142_ovalXY(theObject->color,x-rand()%3,y-rand()%3,
								  x+CR+rand()%3,y-CR+rand()%3,FILL);
	GP142_ovalXY(theObject->color,x+rand()%3,y+rand()%3,
								  x-CR+rand()%3,y+CR+rand()%3,FILL);
	GP142_ovalXY(theObject->color,x+rand()%3,y+rand()%3,
								  x-CR+rand()%3,y-CR+rand()%3,FILL);

	GP142_ovalXY(theObject->color,x-CR-CD-rand()%3,y+CR/2+rand()%3,
								  x-CD+rand()%3,y-CR/2+rand()%3,FILL);
	GP142_ovalXY(theObject->color,x+CR+CD+rand()%3,y+CR/2+rand()%3,
								  x+CD-rand()%3,y-CR/2+rand()%3,FILL);

}

// ----------------------------------------------------------------//

void handleMouseClick(point mPt, card *theCard, status *theStatus) {
	int mouse_click;
	mouse_click = classifyMouseClick(mPt);
			
	if (mouse_click == CLICK_BUTTON) {
				
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/
		handleButton(mPt,theCard,theStatus);

	} else if (mouse_click == CLICK_CARD) {
		//move it
		if(theStatus->mode==MOVEOBJMODE) {
			handleMove(mPt,theStatus,theCard);
		}
		//add a shape
		if(theStatus->mode==ADDOBJMODE) {
			addShape(mPt,theCard,theStatus);
			theStatus->mode=MOVEOBJMODE;
		}
		// add a message
		if(theStatus->mode==ADDMSGMODE) {
			addMsg(mPt,theCard,theStatus);
			theStatus->mode=MOVEOBJMODE;
		}
		//select it
		if(theStatus->mode==SELMODE) {
			handleSelect(mPt,theStatus,theCard);
		}
		
	}
	/* Ignore any other mouse clicks */
}

void handleButton(point mPt, card *theCard, status *theStatus) {
	int buttonNum = 0;
	
	/* If we're outside of this area, we could not have clicked on
	* a button. */
	if (mPt.x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
	//which button
	buttonNum = (GP142_YMAX - mPt.y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
		case BTN_BGCOLOR: // Change the background color
			theStatus->mode = BGCOLORMODE;
			theCard->bgColor = (theCard->bgColor + 1) % MAX_COLORS;
			break;
		case BTN_OBJCOLOR:
			theStatus->mode = OBJCOLORMODE;
			if(theCard->cObj >= 0) {
				theCard->cdObject[theCard->cObj].color =
				(theCard->cdObject[theCard->cObj].color+1)
				%MAX_COLORS;
			}
			break;
		case BTN_ADDOBJ:
			theStatus->mode = ADDOBJMODE;
			break;
		case BTN_ADDMSG:
			theStatus->mode = ADDMSGMODE;
			break;
		case BTN_NEXT: // Change the type of the currently selected object
			if (theCard->cObj >= 0) {
				if(theCard->cdObject[theCard->cObj].type < FIRST_MSG) {
					theCard->cdObject[theCard->cObj].type = 
					(theCard->cdObject[theCard->cObj].type+1)%MAX_TYPES;
				} else {
					theCard->cdObject[theCard->cObj].type = 
					((theCard->cdObject[theCard->cObj].type+1)-FIRST_MSG)
					%MAX_MSGS+FIRST_MSG;
				}
			}
			break;
		case BTN_SEL:
			theStatus->mode = SELMODE;
			break;
		case BTN_FRACTAL:
			theStatus->mode = FRACTALMODE;
			break;
		default:
			theStatus->mode = 999;
			/*Bad news if this is ever reached! */
			break;
	}
}

#define CLICKACCURACY		20
void handleSelect(point mPt, status *theStatus, card *theCard) {
	int i,x,y;
	for(i=0;i<=theCard->numObjects;i++) {
		if(theCard->cdObject[i].type < FIRST_MSG) {
			x = theCard->cdObject[i].oPt.x;
			y = theCard->cdObject[i].oPt.y;
			if( (x > mPt.x-CLICKACCURACY) &&
				(x < mPt.x+CLICKACCURACY) &&
				(y > mPt.y-CLICKACCURACY) &&
				(y < mPt.y+CLICKACCURACY) ) {
				theCard->cObj = i;
				theStatus->mode = MOVEOBJMODE;
				i = theCard->numObjects+1;		//exit loop
			}
		} else {	//a message, don't have to move those 
					//nothing to do, yippee
		} 
	}
}

void handleMove(point mPt, status *theStatus, card *theCard) {
	if(theCard->cObj >= 0) {
		theCard->cdObject[theCard->cObj].oPt.x = mPt.x;
		theCard->cdObject[theCard->cObj].oPt.y = mPt.y;
	}
}

int classifyMouseClick(point mPt) {
	if ((mPt.x < -GP142_XMAX + BUTTON_WIDTH) &&
		(mPt.y < GP142_YMAX) &&
		(mPt.y > (GP142_YMAX - num_buttons*BUTTON_HEIGHT))) {
		return CLICK_BUTTON;
	} else if (mPt.x >= PAGE_LLX && mPt.x < PAGE_LLX + PAGE_WIDTH
		&& mPt.y >= PAGE_LLY && mPt.y < PAGE_LLY + PAGE_HEIGHT) {
		return CLICK_CARD;
	} else {
		return CLICK_OUTSIDE;
	}
}

//-----------------------------------------------------------------//

void addShape(point mPt, card *theCard, status *theStatus) {
	theCard->numObjects++;
	theCard->cObj = theCard->numObjects;

	theStatus->mode = MOVEOBJMODE;
	
	initObj(&theCard->cdObject[theCard->cObj],mPt,AOBJ);
}

void addMsg(point mPt, card *theCard, status *theStatus) {
	theCard->numObjects++;
	theCard->cObj = theCard->numObjects;

	theStatus->mode = MOVEOBJMODE;

	initObj(&theCard->cdObject[theCard->cObj],mPt,AMSG);
}

//-----------------------------------------------------------------//

void initCard(card *theCard) {
	theCard->bgColor	= RED;
	theCard->numObjects = -1;
	theCard->cObj		= 0;
}

void initObj(object *theObject, point mPt,int type) {
	if(type==AOBJ) {
		theObject->type		= HEART_SHAPE;
	} else {
		theObject->type		= FIRST_MSG;
	}
	theObject->color	= BLACK;
	theObject->oPt.x	= mPt.x;
	theObject->oPt.y	= mPt.y;
	theObject->var		= 0;
}

//-----------------------------------------------------------------//

//---------------------- Fractals.c -------------------------------//

#define MINBRANCHES			2
#define MAXBRANCHES			5
#define LOTSOBRANCHES		10
#define MINITERS			5
#define	MAXITERS			8
#define MINANGLE			5
#define MAXANGLE			175
#define	MINLENGTH			50
#define MAXLENGTH			90

#define MAXTRIITERS			32

#define FUDGE				40
#define UPDATERATE			15

#define LINES				1
#define POINTS				0

#define PI					3.14159265359

typedef struct {
	double	angle;
	double	length;
} branch;

typedef struct {
	int		numBranches;
	int		x,y;
	int		iters;
	double	angle;
	double	length;
	branch	branch[LOTSOBRANCHES];
} tree;

typedef struct {
	double	angle[2];
	double	length1,length2,length3;
	double	ratio[2];
	int		iters;
	point	p1,p2,p3;
} triangle;

//-----------------------------------------------------------------//

extern HDC offscreen_DC, the_hdc;

//-----------------------------------------------------------------//

//void	drawFractalTree(void);
void	drawLimb(int x1, int y1, int x2, int y2,int color);
void	iterateBranch(int x, int y, int iter, int bFlag,
					  double angle, double length, tree *theTree);
void	initBranches(tree *theTree);
void	initTree(tree *theTree);

//void	drawFractalTriangle(void);
void	iterateTriangle(point p1, point p2, int iters, int leg, 
						int bFlag, triangle *theTriangle);
void	drawTriangle(point p1, point p2, point p3, int flag, int color);
void	initTriangle(triangle *theTriangle);
double	distPoints(point p1, point p2);
double	anglePoints(point p1, point p2);

void	updateNOW(void);
int		kbHIT(void);

//-----------------------------------------------------------------//

void drawFractalTree(void) {
	tree	theTree;
	int		stemX,stemY;

	GP142_printfXY(RED,-230,-220,9,"Hit a key (twice) to stop.");
	GP142_printfXY(RED,-230,-230,9,"Click the mouse to draw another.");

	initTree(&theTree);
	initBranches(&theTree);

	stemX = theTree.x+(int)(cos(theTree.angle)*theTree.length);
	stemY = theTree.y+(int)(sin(theTree.angle)*theTree.length);

	drawLimb(theTree.x,theTree.y,stemX,stemY,theTree.iters);
	iterateBranch(stemX,stemY,0,1, 
					theTree.angle,theTree.length,&theTree);

}

void iterateBranch(int x, int y, int iter, int bFlag,
	/************/ double angle, double length, tree *theTree) {
	int		i,limbX,limbY;
	double	limbAngle,limbLength;
	static	int j = 0;
	static	int flag = 1;

	if(bFlag) {
		flag = 1;
	}

	if( iter<(theTree->iters) ) {
		for(i=0;i<(theTree->numBranches);i++) {
			
			if(kbHIT()) {
				iter = theTree->iters;
				i = theTree->numBranches;
				flag = 0;
			} //doesn't work
			
			limbAngle	= angle+(theTree->branch[i].angle)-(PI/2.0);
			limbLength	= length*(theTree->branch[i].length);
			limbX		= x+(int)(cos(limbAngle)*limbLength);
			limbY		= y+(int)(sin(limbAngle)*limbLength);
			drawLimb(x,y,limbX,limbY,iter);
			if( (iter+1)<=(theTree->iters) && flag) {
				iterateBranch(limbX,limbY,iter+1,0,
							  limbAngle,limbLength,theTree);
			}
			if(j%UPDATERATE == 0) {
				updateNOW();
			}
			j++;
		}
	}
	
}

void drawLimb(int x1, int y1, int x2, int y2, int color) {
	int colors[12] = {14,19,20,21,19,20,21,19,20,21,19,20};
	
	if(color < 12) {
		color = colors[color];
	} else {
		color = colors[11];
	}
	GP142_lineXY(color,x1,y1,x2,y2,1);
}

void initTree(tree *theTree) {
	time_t t;

	srand(time(&t));

	theTree->numBranches	= (rand()%(MAXBRANCHES-MINBRANCHES+1))+MINBRANCHES;
	theTree->x				= 0;
	theTree->y				= -190;
	theTree->angle			= PI/2.0;
	theTree->length			= 120.0;
	theTree->iters			= (rand()%(MAXITERS-MINITERS+1))+MINITERS;

	if( (theTree->iters)>MAXITERS ) {
		theTree->iters = MAXITERS;
	}
}

void initBranches(tree *theTree) {
	int i;

	if(theTree->numBranches > MAXBRANCHES ) { 
		theTree->numBranches = MAXBRANCHES-1;
	}

	for(i=0;i<(theTree->numBranches);i++) {
		theTree->branch[i].angle	=			//angle in min to max
			((double)((rand()%(MAXANGLE-MINANGLE+1))+MINANGLE))/180.0*PI;
		theTree->branch[i].length	= 			//% from min to max
			((double)((rand()%(MAXLENGTH-MINLENGTH+1))+MINLENGTH))/100.0;		
	}
}

//-----------------------------------------------------------------//

void drawFractalTriangle(void) {
	triangle	theTriangle;

	initTriangle(&theTriangle);

	GP142_printfXY(RED,-230,-220,10,"Hit a key (twice) to stop.");
	GP142_printfXY(RED,-230,-230,10,"Click the mouse to draw another.");

	drawTriangle(theTriangle.p1,theTriangle.p2,theTriangle.p3,1,7);
	iterateTriangle(theTriangle.p1,theTriangle.p3,0,0,1,&theTriangle);
	iterateTriangle(theTriangle.p3,theTriangle.p2,0,1,1,&theTriangle);
}

void iterateTriangle(point p1, point p2, int iters, int leg, int bFlag,
	/*************/	 triangle *theTriangle) {
	point		p3,tp;
	double		angle,distance;
	static	int j = 0;
	static	int flag = 1;

	if(bFlag) {
		flag = 1;
	}

	angle		=	anglePoints(p1,p2);
	angle		+=	theTriangle->angle[0];
	distance	=	distPoints(p1,p2); 

	tp.x = p1.x; tp.y = p1.y;

	p3.x = (int) (cos(angle)*(theTriangle->ratio[0]*distance))+tp.x;
	p3.y = (int) (sin(angle)*(theTriangle->ratio[0]*distance))+tp.y;
	
	drawTriangle(p1,p2,p3,POINTS,iters);

	if(!(j%(UPDATERATE*5))) {
		updateNOW();
	}
	j++;
	
	if(kbHIT()) {
		flag = 0;
	}

	if( !(distPoints(p1,p3) < 1.4) && flag && (iters < MAXTRIITERS) ) {
		iterateTriangle(p1,p3,iters+1,0,0,theTriangle);
	}
	if( !(distPoints(p3,p2) < 1.4) && flag && (iters < MAXTRIITERS) ) {
		iterateTriangle(p3,p2,iters+1,1,0,theTriangle);
	}

}

void initTriangle(triangle *theTriangle) {
	theTriangle->p1.x = -120;	theTriangle->p1.y = -60;
	theTriangle->p2.x = 120;	theTriangle->p2.y = -60;

	theTriangle->p3.x = (rand()%241)+0	+ theTriangle->p1.x;
	theTriangle->p3.y = (rand()%121)+0	+ theTriangle->p1.y;

	theTriangle->length3 = distPoints(theTriangle->p1,theTriangle->p2);
	theTriangle->length1 = distPoints(theTriangle->p1,theTriangle->p3);
	theTriangle->length2 = distPoints(theTriangle->p2,theTriangle->p3);

	theTriangle->angle[0] = anglePoints(theTriangle->p1,theTriangle->p3);
	theTriangle->angle[1] = anglePoints(theTriangle->p2,theTriangle->p3);

	theTriangle->iters	  = MAXTRIITERS;

	theTriangle->ratio[0] = theTriangle->length1/theTriangle->length3;
	theTriangle->ratio[1] = theTriangle->length2/theTriangle->length3;
}

void drawTriangle(point p1, point p2, point p3, int flag, int color) {
	int colors[32] = {22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,
					  5,4,3,4,5,6,7,8,9,10,11,12,13,14,15};
	if(flag) {
		GP142_lineXY(colors[color],p1.x,p1.y,p2.x,p2.y,1);
		GP142_lineXY(colors[color],p2.x,p2.y,p3.x,p3.y,1);
		GP142_lineXY(colors[color],p1.x,p1.y,p3.x,p3.y,1);
	} else {
		GP142_pixelXY(colors[color],p3.x,p3.y);
	}
}

double distPoints(point p1, point p2) {
	return sqrt(((double)
				((p1.x-p2.x)*(p1.x-p2.x)+(p1.y-p2.y)*(p1.y-p2.y))));
}

double anglePoints(point p1, point p2) {
	double angle;
	angle = atan( (double) (p2.y-p1.y) / (double) (p2.x-p1.x) );
	//if( (p2.x<p1.x) && (p2.y<p1.y) ) { angle = angle+(PI/2.0); }
	//if( (p2.x<p1.x) && (p2.y>p1.y) ) { angle = angle+(PI/2.0); }
	if( (p2.x<p1.x) ) { angle = angle+(PI); }
	if(angle>( 2.0*PI)) { angle += (-2.0*PI); }
	if(angle<(-2.0*PI)) { angle += ( 2.0*PI); }
	return angle;
}

//-----------------------------------------------------------------//

void updateNOW(void) {
	if(the_hdc && offscreen_DC) {
		BitBlt (the_hdc,0,0,2*GP142_XMAX+1, 2*GP142_YMAX+FUDGE,
			offscreen_DC,0,0,SRCCOPY);
	}
}

int	kbHIT(void) {
	MSG msg;
	int flag = 0;

	PeekMessage(&msg, (HWND)NULL, 0, 0, PM_REMOVE);
	if (msg.message == WM_KEYDOWN) {
		flag = 1;
	}

	return flag;
}
